/*
 * C
 *
 * Copyright 2016 IS2T. All rights reserved.
 * For demonstration purpose only.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/* Includes ------------------------------------------------------------------*/

#include <stdint.h>
#include "LLCOMM_UART.h"
#include "LLCOMM_impl.h"
#include "microej.h"
#include "stm32f4xx_hal_rcc.h"
#include "FreeRTOS.h"
#include "task.h"
#include "semphr.h"
#include "interrupts.h"

/* Methods declaration -------------------------------------------------------*/

void LLCOMM_UART_new(LLCOMM_BUFFERED_CONNECTION* env);

/* Enumerations --------------------------------------------------------------*/

/*
 * Defines the mapping between the HAL UART instance and MicroEJ LLCOMM_UARTs
 * table. The indexes are 0-based.
 */
typedef enum
{
	UART1_INDEX = 0,
	UART6_INDEX = 1,
} hal_to_microej_uart_table;

/* Fields --------------------------------------------------------------------*/

/*
 * Array of all static LLCOMM UARTs.
 */
static LLCOMM_UART LLCOMM_UARTs[2];

/* Private functions ---------------------------------------------------------*/

/*
 * Initialize LLCOMM_UARTs array for a specific HAL UART instance and initialize
 * corresponding MicroEJ LLCOMM_UART.
 */
static void COMMS_initialize_llcomm_uart(int32_t index, USART_TypeDef* hal_uart_instance, uint8_t platform_id)
{
	// store HAL UART instance
	LLCOMM_UARTs[index].hal_uart_handle.Instance = hal_uart_instance;

	// store HAL UART ID
	LLCOMM_UARTs[index].platformId = platform_id;

	// create MicroEJ UART connection
	LLCOMM_UART_new(&LLCOMM_UARTs[index].header);

	// add MicroEJ UART connection into MicroEJ framework as generic connection
	LLCOMM_addConnection((LLCOMM_CONNECTION*)&LLCOMM_UARTs[index].header);
}

static void COMMS_initialize_interrupt(IRQn_Type interrupt)
{
	HAL_NVIC_SetPriority(interrupt, 0xF, 0xF);
	HAL_NVIC_EnableIRQ(interrupt);
}

static void COMMS_initialize_pin(GPIO_TypeDef* port, uint16_t pin, uint32_t alternate)
{
	GPIO_InitTypeDef GPIO_InitStructure;
	GPIO_InitStructure.Mode = GPIO_MODE_AF_PP;
	GPIO_InitStructure.Pull = GPIO_PULLUP;
	GPIO_InitStructure.Speed = GPIO_SPEED_HIGH;
	GPIO_InitStructure.Alternate = alternate;
	GPIO_InitStructure.Pin = pin;
	HAL_GPIO_Init(port, &GPIO_InitStructure);
}

/* Interrupt functions -------------------------------------------------------*/

void USART1_IRQHandler(void)
{
	LLCOMM_UART_callback(&LLCOMM_UARTs[UART1_INDEX]);
}

void USART6_IRQHandler(void)
{
	LLCOMM_UART_callback(&LLCOMM_UARTs[UART6_INDEX]);
}

/* Public functions ----------------------------------------------------------*/

void LLCOMM_IMPL_initialize(void)
{		
	// STM32F412G-DISCO COM virtual port 1: use USART 1
	// 	- TX: PA15	P2-40
	// 	- RX: PA10	P2-41
	__HAL_RCC_USART1_CLK_ENABLE();
	__HAL_RCC_GPIOA_CLK_ENABLE();
	COMMS_initialize_pin(GPIOA, GPIO_PIN_15, GPIO_AF7_USART1);	// TX
	COMMS_initialize_pin(GPIOA, GPIO_PIN_10, GPIO_AF7_USART1);	// RX
	COMMS_initialize_llcomm_uart(UART1_INDEX, USART1, 1);
	COMMS_initialize_interrupt(USART1_IRQn);
	
	// STM32F412G-DISCO COM virtual port 2: use USART 6
	// 	- TX: PG14	CN11-2
	// 	- RX: PG9	CN11-1
	__HAL_RCC_USART6_CLK_ENABLE();
	__HAL_RCC_GPIOG_CLK_ENABLE();
	COMMS_initialize_pin(GPIOG, GPIO_PIN_14, GPIO_AF8_USART6);	// TX
	COMMS_initialize_pin(GPIOG, GPIO_PIN_9, GPIO_AF8_USART6);	// RX
	COMMS_initialize_llcomm_uart(UART6_INDEX, USART6, 6);
	COMMS_initialize_interrupt(USART6_IRQn);	
}

void LLCOMM_IMPL_syncConnectionsEnter(void)
{
	// no dynamic connection: nothing to do
}

void LLCOMM_IMPL_syncConnectionsExit(void)
{
	// no dynamic connection: nothing to do
}

