/*
 * C
 *
 * Copyright 2016 IS2T. All rights reserved.
 * For demonstration purpose only.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/* Includes ------------------------------------------------------------------*/

#include "LLDISPLAY_configuration.h"
#include "LLDISPLAY_st7789h2.h"
#include "LLDISPLAY_IMPL.h"
#include "framerate.h"
#include "bsp_util.h"

/* Defines -------------------------------------------------------------------*/

/*
 * Declare MicroEJ Display buffers
 */
BSP_DECLARE_BUFFER(DISPLAY_MEM)
#define BUFFER_SIZE (ST7789H2_LCD_PIXEL_WIDTH * ST7789H2_LCD_PIXEL_HEIGHT * LCD_BPP / 8)
#define FRAME_BUFFER (DISPLAY_MEM_START)
#define WORKINGBUFFER_START (FRAME_BUFFER + BUFFER_SIZE)
#define WORKINGBUFFER_END (DISPLAY_MEM_END)

/* Private functions ---------------------------------------------------------*/

/*
 * Initialize LCD tearing pin
 */
static void LLDISPLAY_init_tearing(void)
{
	GPIO_InitTypeDef gpio_init_structure;
	__HAL_RCC_GPIOG_CLK_ENABLE();
	gpio_init_structure.Pin = LCD_TEARING_PIN;
	gpio_init_structure.Pull = GPIO_NOPULL;
	gpio_init_structure.Speed = GPIO_SPEED_FAST;
	gpio_init_structure.Mode = GPIO_MODE_IT_RISING;
	HAL_GPIO_Init(LCD_TEARING_PORT, &gpio_init_structure);
	HAL_NVIC_SetPriority(LCD_TEARING_IRQ, 0x0F, 0x00);
	HAL_NVIC_EnableIRQ(LCD_TEARING_IRQ);
}

/* Interrupt -----------------------------------------------------------------*/

/*
 * LCD tearing interrupt
 */
void EXTI4_IRQHandler(void)
{
	// clear interrupt
	__HAL_GPIO_EXTI_CLEAR_IT(LCD_TEARING_PIN);

	LLDISPLAY_tearing_interrupt();
}

/* API -----------------------------------------------------------------------*/

/*
 * Initialize the display.
 */
void LLDISPLAY_IMPL_initialize(void)
{
	
#if LCD_ORIENTATION == LCD_ORIENTATION_LANDSCAPE_1
		BSP_LCD_InitEx(LCD_ORIENTATION_LANDSCAPE);
#elif	LCD_ORIENTATION == LCD_ORIENTATION_PORTRAIT_1
	BSP_LCD_InitEx(LCD_ORIENTATION_PORTRAIT);
#endif
	
	LLDISPLAY_init_tearing();
	LLDISPLAY_initialization();
}

/*
 * Return LCD width
 */
int32_t LLDISPLAY_IMPL_getWidth(void)
{
	return ST7789H2_LCD_PIXEL_WIDTH;
}

/*
 * Return LCD height
 */
int32_t LLDISPLAY_IMPL_getHeight(void)
{
	return ST7789H2_LCD_PIXEL_HEIGHT;
}

/*
 * Returns the start address of the buffer used to store the display stack's heap.
 */
int32_t LLDISPLAY_IMPL_getWorkingBufferStartAddress(void)
{
	return (uint32_t)WORKINGBUFFER_START;
}

/*
 * Returns the end address of the buffer used to store the display stack's heap.
 */
int32_t LLDISPLAY_IMPL_getWorkingBufferEndAddress(void)
{
	return (uint32_t)WORKINGBUFFER_END;
}

/*
 * Returns the address of the back buffer. This buffer will be used by the
 * display stack to draw application drawings.
 * @return the buffer address
 */
int32_t LLDISPLAY_IMPL_getGraphicsBufferAddress(void)
{
	return (int32_t)FRAME_BUFFER;
}

/*
 * Synchronizes the display stack drawing with the display framerate.<br>
 * This method is called by the display stack before the very first drawing
 * after a flush. This allows to the display driver to wait for the right
 * moment to allow the display stack to start using the back buffer
 * in order to avoid flickering.<br>
 */
void LLDISPLAY_IMPL_synchronize(void)
{
	// wait the end of previous copy
	Wait_LCD();
}

/*
 * Performs a flush: the content of given square in back buffer must be
 * copied into LCD buffer.
 */
int32_t LLDISPLAY_IMPL_flush(int32_t addr, int32_t xmin, int32_t ymin, int32_t xmax, int32_t ymax)
{

#ifdef FRAMERATE_ENABLED
	framerate_increment();
#endif

	// prepare LCD
	ST7789H2_SetCursor(0, ymin);
	ST7789H2_WriteReg(ST7789H2_WRITE_RAM, (uint8_t*)NULL, 0);   /* RAM write data command */

	// prepare copy
	int32_t size = (ymax - ymin + 1) * ST7789H2_LCD_PIXEL_WIDTH;	// size in pixels
	int32_t src = addr + ymin * ST7789H2_LCD_PIXEL_WIDTH * LCD_BPP / 8;

	LLDISPLAY_copy_buffer(src, size);

	return addr;
}
