/*
 * C
 *
 * Copyright 2016 IS2T. All rights reserved.
 * For demonstration purpose only.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/*
 * This file converts the input events in MicroUI Event Generator events.
 */

/* Includes ------------------------------------------------------------------*/

#include "LLINPUT.h"
#include "stm32412g_discovery.h"
#include "microej.h"

// this h file is created by buildSystemMicroUI step
#include "microui_constants.h"

/* Defines -------------------------------------------------------------------*/

/*
 * Defines the three modes of joystick buttons
 * When a button is pressed/released...
 *
 * - BUTTON_SEND_BUTTON_EVENT: a MicroUI Buttons event is send (press / repeat / release)
 * - BUTTON_SEND_COMMAND_EVENT: a MicroUI Command event is send (press / repeat only )
 *
 * The Buttons and Command events are sent to the MicroUI application.
 * The Home event is reserved to the internal framework.
 */
#define JOYSTICK_SEND_BUTTON_EVENT	(0)
#define JOYSTICK_SEND_COMMAND_EVENT	(1)

/*
 * Choose the button mode selected a value in previous list
 */
#define JOYSTICK_SEND_EVENT JOYSTICK_SEND_COMMAND_EVENT

/* Private functions ---------------------------------------------------------*/

/*
 * Specify which MicroUI Command to send when the mode "BUTTON_SEND_COMMAND_EVENT" is selected
 */
#if JOYSTICK_SEND_EVENT == JOYSTICK_SEND_COMMAND_EVENT
static uint8_t EVENT_GENERATOR_send_command(int32_t buttonId)
{
	switch(buttonId)
	{
	case JOY_UP: 	return LLINPUT_sendCommandEvent(MICROUI_EVENTGEN_COMMANDS, LLINPUT_COMMAND_UP);
	case JOY_DOWN: 	return LLINPUT_sendCommandEvent(MICROUI_EVENTGEN_COMMANDS, LLINPUT_COMMAND_DOWN);
	case JOY_LEFT: 	return LLINPUT_sendCommandEvent(MICROUI_EVENTGEN_COMMANDS, LLINPUT_COMMAND_LEFT);
	case JOY_RIGHT:	return LLINPUT_sendCommandEvent(MICROUI_EVENTGEN_COMMANDS, LLINPUT_COMMAND_RIGHT);
	case JOY_SEL: 	return LLINPUT_sendCommandEvent(MICROUI_EVENTGEN_COMMANDS, LLINPUT_COMMAND_SELECT);
	}
	return MICROEJ_FALSE;	// invalid joystick button ID
}
#endif

/* Buttons -------------------------------------------------------------------*/

uint8_t EVENT_GENERATOR_button_pressed(int32_t buttonId)
{
#if JOYSTICK_SEND_EVENT == JOYSTICK_SEND_BUTTON_EVENT
	// send a MicroUI Buttons "press" event
	return LLINPUT_sendButtonPressedEvent(MICROUI_EVENTGEN_JOYSTICK, buttonId);
#else // JOYSTICK_SEND_EVENT == JOYSTICK_SEND_COMMAND_EVENT
	// send a MicroUI Command event
	return EVENT_GENERATOR_send_command(buttonId);
#endif
}

uint8_t EVENT_GENERATOR_button_repeated(int32_t buttonId)
{
#if JOYSTICK_SEND_EVENT == JOYSTICK_SEND_BUTTON_EVENT
	// send a MicroUI Buttons "release" event
	return LLINPUT_sendButtonRepeatedEvent(MICROUI_EVENTGEN_JOYSTICK, buttonId);
#else // JOYSTICK_SEND_EVENT == JOYSTICK_SEND_COMMAND_EVENT
	// send a MicroUI Command event
	return EVENT_GENERATOR_send_command(buttonId);
#endif
}

uint8_t EVENT_GENERATOR_button_released(int32_t buttonId)
{
#if JOYSTICK_SEND_EVENT == JOYSTICK_SEND_BUTTON_EVENT
	// send a MicroUI Buttons "release" event
	return LLINPUT_sendButtonReleasedEvent(MICROUI_EVENTGEN_JOYSTICK, buttonId);
#else
	// do not send a Command event on release event
	return MICROEJ_TRUE; // the event has been managed
#endif
}

/* Touch ---------------------------------------------------------------------*/

#include <stdio.h>
uint8_t EVENT_GENERATOR_touch_pressed(int32_t x, int32_t y)
{
	return LLINPUT_sendTouchPressedEvent(MICROUI_EVENTGEN_TOUCH, x, y);
}

uint8_t EVENT_GENERATOR_touch_moved(int32_t x, int32_t y)
{
	return LLINPUT_sendTouchMovedEvent(MICROUI_EVENTGEN_TOUCH, x, y);
}

uint8_t EVENT_GENERATOR_touch_released(void)
{
	return LLINPUT_sendTouchReleasedEvent(MICROUI_EVENTGEN_TOUCH);
}
