
/*
 *	C
 *	
 *	Copyright 2016 IS2T. All rights reserved.
 *	IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/* Includes ------------------------------------------------------------------*/

#include "stm32412g_discovery.h"
#include "interrupts.h"
#include "buttons_helper.h"
#include "LLDISPLAY_configuration.h"

/* Private functions ---------------------------------------------------------*/

static void JOYSTICK_MANAGER_interrupt(GPIO_TypeDef * GPIO_Port, uint16_t GPIO_Pin, JOYState_TypeDef type)
{
	uint8_t leave = interrupt_enter();

	if (HAL_GPIO_ReadPin(GPIO_Port, GPIO_Pin) == GPIO_PIN_SET)
	{
		BUTTONS_HELPER_pressed((int32_t)type);
	}
	else
	{
		BUTTONS_HELPER_released((int32_t)type);
	}
	interrupt_leave(leave);
}

/* Interrupts ----------------------------------------------------------------*/

void EXTI0_IRQHandler(void)
{
	__HAL_GPIO_EXTI_CLEAR_IT(GPIO_PIN_0);
	JOYSTICK_MANAGER_interrupt(SEL_JOY_GPIO_PORT, SEL_JOY_PIN, JOY_SEL);
#if LCD_ORIENTATION == LCD_ORIENTATION_LANDSCAPE_1
		JOYSTICK_MANAGER_interrupt(UP_JOY_GPIO_PORT, UP_JOY_PIN, JOY_UP);
#elif	LCD_ORIENTATION == LCD_ORIENTATION_PORTRAIT_1
		JOYSTICK_MANAGER_interrupt(UP_JOY_GPIO_PORT, UP_JOY_PIN, JOY_LEFT);
#endif
}

void EXTI1_IRQHandler(void)
{
	__HAL_GPIO_EXTI_CLEAR_IT(GPIO_PIN_1);
#if LCD_ORIENTATION == LCD_ORIENTATION_LANDSCAPE_1
		JOYSTICK_MANAGER_interrupt(UP_JOY_GPIO_PORT, UP_JOY_PIN, JOY_DOWN);
#elif	LCD_ORIENTATION == LCD_ORIENTATION_PORTRAIT_1
		JOYSTICK_MANAGER_interrupt(UP_JOY_GPIO_PORT, UP_JOY_PIN, JOY_RIGHT);
#endif
}

void EXTI15_10_IRQHandler(void)
{
	if(__HAL_GPIO_EXTI_GET_IT(LEFT_JOY_PIN) != RESET)
	{
		__HAL_GPIO_EXTI_CLEAR_IT(LEFT_JOY_PIN);
#if LCD_ORIENTATION == LCD_ORIENTATION_LANDSCAPE_1
		JOYSTICK_MANAGER_interrupt(UP_JOY_GPIO_PORT, UP_JOY_PIN, JOY_LEFT);
#elif	LCD_ORIENTATION == LCD_ORIENTATION_PORTRAIT_1
		JOYSTICK_MANAGER_interrupt(UP_JOY_GPIO_PORT, UP_JOY_PIN, JOY_DOWN);
#endif
	}

	if(__HAL_GPIO_EXTI_GET_IT(RIGHT_JOY_PIN) != RESET)
	{
		__HAL_GPIO_EXTI_CLEAR_IT(RIGHT_JOY_PIN);
#if LCD_ORIENTATION == LCD_ORIENTATION_LANDSCAPE_1
		JOYSTICK_MANAGER_interrupt(UP_JOY_GPIO_PORT, UP_JOY_PIN, JOY_RIGHT);
#elif	LCD_ORIENTATION == LCD_ORIENTATION_PORTRAIT_1
		JOYSTICK_MANAGER_interrupt(UP_JOY_GPIO_PORT, UP_JOY_PIN, JOY_UP);
#endif
	}
}

/* API -----------------------------------------------------------------------*/

void JOYSTICK_MANAGER_enable_interrupts(void)
{
	HAL_NVIC_EnableIRQ(SEL_JOY_EXTI_IRQn);
	HAL_NVIC_EnableIRQ(UP_JOY_EXTI_IRQn);
	HAL_NVIC_EnableIRQ(DOWN_JOY_EXTI_IRQn);
	HAL_NVIC_EnableIRQ(LEFT_JOY_EXTI_IRQn);
	HAL_NVIC_EnableIRQ(RIGHT_JOY_EXTI_IRQn);
}

void JOYSTICK_MANAGER_disable_interrupts(void)
{
	HAL_NVIC_DisableIRQ(SEL_JOY_EXTI_IRQn);
	HAL_NVIC_DisableIRQ(UP_JOY_EXTI_IRQn);
	HAL_NVIC_DisableIRQ(DOWN_JOY_EXTI_IRQn);
	HAL_NVIC_DisableIRQ(LEFT_JOY_EXTI_IRQn);
	HAL_NVIC_DisableIRQ(RIGHT_JOY_EXTI_IRQn);
}

void JOYSTICK_MANAGER_initialize(void)
{
	BUTTONS_HELPER_initialize();
	BSP_JOY_Init(JOY_MODE_EXTI);
	JOYSTICK_MANAGER_disable_interrupts();
}
