/*
 * C
 *
 * Copyright 2016 IS2T. All rights reserved.
 * For demonstration purpose only.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/* Includes ------------------------------------------------------------------*/

#include "microej.h"
#include "io_task.h"
#include "touch_helper.h"
#include "st7789h2.h"
#include "stm32412g_discovery_ts.h"
#include "stm32412g_discovery_lcd.h"
#include "LLDISPLAY_configuration.h"

/* Global --------------------------------------------------------------------*/

static uint8_t touch_initialized = MICROEJ_FALSE;

/* Private API ---------------------------------------------------------------*/

static uint8_t TOUCH_MANAGER_update(void)
{
	TS_StateTypeDef TS_State;
	BSP_TS_GetState(&TS_State);

	if (TS_State.touchDetected)
	{
		// here, pen is down for sure
#if LCD_ORIENTATION == LCD_ORIENTATION_LANDSCAPE_1
		TOUCH_HELPER_pressed(TS_State.touchX[0], TS_State.touchY[0]);
#elif	LCD_ORIENTATION == LCD_ORIENTATION_PORTRAIT_1
		TOUCH_HELPER_pressed(TS_State.touchY[0], ST7789H2_LCD_PIXEL_WIDTH - TS_State.touchX[0]);
#endif
		return MICROEJ_TRUE;
	}
	else
	{
		// here, pen is up for sure
		TOUCH_HELPER_released();
		return MICROEJ_FALSE;
	}
}

/* API -----------------------------------------------------------------------*/

void TOUCH_MANAGER_initialize(void)
{
	BSP_TS_Init(ST7789H2_LCD_PIXEL_WIDTH, ST7789H2_LCD_PIXEL_HEIGHT);
	BSP_TS_ITConfig();
	IO_TASK_create_task();
	touch_initialized = MICROEJ_TRUE;
}

uint8_t TOUCH_MANAGER_interrupt(void)
{
	if(__HAL_GPIO_EXTI_GET_IT(TS_INT_PIN) != RESET)
	{
		__HAL_GPIO_EXTI_CLEAR_IT(TS_INT_PIN);

		// disable the EXTI interrupt until next touch release
		HAL_NVIC_DisableIRQ(TS_INT_EXTI_IRQn);

		return MICROEJ_TRUE;
	}

	return MICROEJ_FALSE;
}

uint8_t TOUCH_MANAGER_work(void)
{
	if (touch_initialized == MICROEJ_FALSE)
	{
		return MICROEJ_FALSE;
	}

	// manage touch event
	return TOUCH_MANAGER_update();
}
